#ifndef Thread_H
#define Thread_H

#include "Array.h"
#include "ReferenceCounter.h"

namespace Framework
{
    class Thread;
    class Critical;

    //! Ein neuer Thread wie die Thread Klasse aus Java
    class Thread : public virtual ReferenceCounter
    {
    private:
        pthread_t *threadHandleSys;
        int lockCount;
    protected:
        unsigned long threadId;
        pthread_t threadHandle;
        bool run;

    public:
        //! Konstruktor 
        DLLEXPORT Thread();
        //! Destruktor 
        DLLEXPORT virtual ~Thread();
        //! Startet den neuen Thread 
        DLLEXPORT void start();
#ifdef WIN32
        //! Pausiert den Thread (Nur fr Windows)
        DLLEXPORT void pause();
        //! Setzt den Thread Fort (Nur fr Windows)
        DLLEXPORT void fortsetzen();
#endif
        //! Beendet den Thread
        DLLEXPORT void ende();
        //! Diese Funktion wird von dem neuen Thread ausgefhrt.
        DLLEXPORT virtual void thread();
        //! Diese Funktion wird eufgerufen, nachdem die thread Funktion zuende ist
        DLLEXPORT virtual void threadEnd();
        //! prft, ob der Thrad aktiv ist
        //! return: true, falls der Thread luft. 
        //!         false, wenn der Thread beendet, pausiert oder noch nicht gestartet wurde.
        DLLEXPORT bool isRunning() const;
        //! wartet zeit lang auf den Thread
        //! \param zeit Die Zeit, die auf den Thread gewartet werden soll. 1000 = 1 Sekunde
        DLLEXPORT int warteAufThread( int zeit );
        //! Legt einen Frameworkpointer auf ein Threadhandle fest, der auf 0 gesetzt wird, falls die Ressourcen des Threads bereits follstnfig aufgerumt wurden
        //! \param ths Ein Zeiger auf ein Threadhandle, das verndert werden soll
        void setSystemHandlePointer( pthread_t *ths );
        //! Gibt ein Handle auf den Thread zurck
        DLLEXPORT pthread_t getThreadHandle() const;

    private:
        void addCriticalLock();
        void removeCriticalLock();

    friend Critical;
    };

#ifdef WIN32
    //! Diese Funktion wird fon der Thread Klasse benutzt um einen thread zu starten
    DLLEXPORT unsigned long __stdcall threadStart( void *param );
#else
    //! Diese Funktion wird fon der Thread Klasse benutzt um einen thread zu starten
    void *threadStart( void *param ); //! startet thread
#endif

    //! Eine Klasse, die alle aktuell laufenden Thread Objekte speichert
    class ThreadRegister
    {
    private:
        Array< Thread* > threads;
        CRITICAL_SECTION cs;
        Array< pthread_t > closedThreads;

    public:
        //! Konstruktor
        ThreadRegister();
        //! Destruktor
        ~ThreadRegister();
        //! Fgt einen neuen Thread hinzu
        //! \param t Der Thread, der hinzugefgt werden soll
        void add( Thread *t );
        //! Entfernt einen neuen Thread
        //! \param t Der Thread, der entfernt werden soll
        void remove( Thread *t );
        //! berprft, ob ein Zeiger auf ein gltiges Thread Objekt zeigt, oder ob es schon gelscht wurde
        //! \param t Der Zeiger, der geprft werden soll
        bool isThread( Thread *t );
        //! Sucht nach einem bestimmten Thread und gibt das zugehrige Objekt zurck
        //! handle: Ein handle zu dem gesuchten Thread
        Thread *zThread( pthread_t handle );
        //! Setzt Wird automatisch aufgerufen, wenn ein Thread beendet wird. Die Reccourcen werden daraufhin in cleanUpClosedThreads freigegeben.
        //! \param handle Das Handle des Threads
        void addClosedThread( pthread_t handle );
        //! sperrt das register
        void lock();
        //! entsperrt das register
        void unlock();
        //! Lscht die bereits beendetetn Threads und gibt ihre Reccourcen wieder frei
        DLLEXPORT void cleanUpClosedThreads();
    };
}

#endif